void settick(void);
void disconnect(void);

void setascii(void);
void setbell(void);
void setbinary(void);
void setdebug(int argc, char *argv[]);
void setform(void);
void setglob(void);
void sethash(void);
void setmode(void);
void setport(void);
void setprompt(void);
void setstruct(void);
void settenex(void);
void settrace(void);
void settype(int argc, char *argv[]);
void setverbose(void);
void restart(int argc, char *argv[]);
void reget(int argc, char *argv[]);
void syst(void);
void cd(int argc, char *argv[]);
void lcd(int argc, char *argv[]);
void delete_cmd(int argc, char *argv[]);
void mdelete(int argc, char *argv[]);
void user(int argc, char *argv[]);
void ls(int argc, char *argv[]);
void mls(int argc, char *argv[]);
void get(int argc, char *argv[]);
void mget(int argc, char *argv[]);
void help(int argc, char *argv[]);
void append(int argc, char *argv[]);
void put(int argc, char *argv[]);
void mput(int argc, char *argv[]);
void renamefile(int argc, char *argv[]);
void status(void);
void quote(int argc, char *argv[]);
void rmthelp(int argc, char *argv[]);
void shell(const char *arg);
void site(int argc, char *argv[]);
void pwd(void);
void makedir(int argc, char *argv[]);
void removedir(int argc, char *argv[]);
void setcr(void);
void account(int argc, char *argv[]);
void doproxy(int argc, char *argv[]);
void reset(void);
void setcase(void);
void setntrans(int argc, char *argv[]);
void setnmap(int argc, char *argv[]);
void setsunique(void);
void setrunique(void);
void cdup(void);
void macdef(int argc, char *argv[]);
void sizecmd(int argc, char *argv[]);
void modtime(int argc, char *argv[]);
void newer(int argc, char *argv[]);
void rmtstatus(int argc, char *argv[]);
void do_chmod(int argc, char *argv[]);
void do_umask(int argc, char *argv[]);
void idle_cmd(int argc, char *argv[]);
void setpassive(void);
